@@ -33,7 +33,7 @@ class UrlRewriteNyaa(object):
         advanced.accept('choice', key='filter').accept_choices(FILTERS)
         return root
 
-    def search(self, entry, config):
+    def search(self, task, entry, config):
         if not isinstance(config, dict):
             config = {'category': config}
         config.setdefault('category', 'anime eng')
